#!/usr/bin/env python


## wallpabox.py - GUI - Choose and modify the wallpaper on a fluxbox or openbox desktop
#
# Copyright (C) 2008 Nicolas Doby <nicolas.doby@gmail.com>
#
# Licensed under the GNU General Public License Version 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement


import gtk, pygtk
import os, sys, shutil
import gettext
pygtk.require('2.0')


# Set locales
gettext.install("wallpabox", "/usr/share/locale/")


class ChangeWallpaper:

	command_arg = [" -f ", " -c ", " -t ", " -a ", " -r "]

	# Set the new wallpaper
	def set_wallpaper(self, index):
		# Get the combobox choice selection and add the argumment
		arg = self.command_arg[self.combo.get_active()]
		# Set random mode if the checkbutton is enable
		if(self.randomcheck.get_active() == True):
			arg += self.command_arg[len(self.command_arg)-1]
			os.popen("/usr/bin/fbsetbg"+ arg + self.directory)
			print _("Random wallpapers")
		else:
			# Test if a picture is selected
			if(index != -1):
				filename = "\"" + self.images_paths[index] + "\""
				if(os.path.exists(self.images_paths[index])):
					os.popen("/usr/bin/fbsetbg"+ arg + filename)
					print _("New wallpaper : ") + filename
				else:
					print _("The file \"%s\" does not exist.") % filename
					del self.images_paths[index]
					self.model.remove(self.model.get_iter(index))


	# Callback #

	# Display a 'FileChooserDiablog'. The user can import one or more pictures.
	def select_file(self, widget, filesel):
		result = filesel.run()
		filesel.hide()
		if(result == gtk.RESPONSE_OK):
			filenames = filesel.get_filenames()
			self.add_files_to_iconview(filenames)
			

	# Add files in the iconview
	def add_files_to_iconview(self, filenames):
		for i in filenames:			
			# Check if a filename already exists
			temp = i.split('/')
			new_path = self.directory+temp[len(temp)-1]
			if new_path in self.images_paths:
				print _("The file \"%s\" has already been added.") % i
			else:
				try:
					# Copy the picture in the config folder
					shutil.copy(i, new_path)
					image = gtk.gdk.pixbuf_new_from_file_at_size(new_path, 96, 96)
					print _("Add : ") + i
					self.images_paths.append(new_path)					
					self.model.append([image])
				except:
					print _("Error : %s is not a correct file.") % i

						
	# Set the selected image as the new wallpaper
	def on_apply_click(self, widget, iconview):
		widget.set_sensitive(False)
		liste = iconview.get_selected_items()
		if liste != []:
			index = iconview.get_selected_items()[0][0]
			self.set_wallpaper(index)
		else:
			self.set_wallpaper(-1)

	# Remove all selected images
	def on_remove_click(self, widget, iconview):
		widget.set_sensitive(False)
		liste = iconview.get_selected_items()
		model = iconview.get_model()
		if len(liste) != 0:
			for image in liste:				
				index = image[0]
				filename = self.images_paths[index]					
				model.remove(model.get_iter(index))
				os.remove(filename)
				del self.images_paths[index]
				print _("Remove : ") + filename


	def evnmt_delete(self, widget, evenement, donnees=None):
		return False

	def destroy(self, widget, donnees=None):
		gtk.main_quit()

	# Enable the apply and remove button if the user clicked on a thumbnail
	# Disable them if not
	def on_selection_changed(self, iconview, button_apply, button_remove):
		liste = iconview.get_selected_items()
		if len(liste) != 0:
			if len(liste) == 1:			
				button_apply.set_sensitive(True)
			else:
				button_apply.set_sensitive(False)
			button_remove.set_sensitive(True)
		else:
			if(self.randomcheck.get_active() == False):
				button_apply.set_sensitive(False)
			button_remove.set_sensitive(False)

	# Set the new wallpaper if the user double click on a thumbnail
	def on_double_click(self, iconview, path, button_apply):
		self.randomcheck.set_active(False)
		index = iconview.get_selected_items()[0][0]
		self.set_wallpaper(index)
		button_apply.set_sensitive(False)
		
	# Add wallpapers when drop in the iconview
	def receiveCallback(self, widget, context, x, y, selection, targetType, time):
		# Remove "file://" from the begining and "\r\t" from the end of the string
		data = selection.data[7:len(selection.data)-2]
		# Split the string to have a list of paths
		paths = data.split('\r\nfile://')
		final_paths = []
		# Check if all the files are picture's types
		for p in paths :
			if (p.lower().endswith('jpg') or p.lower().endswith('jpeg') or p.lower().endswith('png') or p.lower().endswith('gif') or p.lower().endswith('bmp')):
				final_paths.append(p)
			else:
				print _("The file \"%s\" is not a valid file.") % p

		self.add_files_to_iconview(final_paths)

	# Enable the aply button if the combobox change
	def on_combo_change(self, widget, button_apply, iconview):
		liste = iconview.get_selected_items()
		if len(liste) != 0:
			button_apply.set_sensitive(True)
		else:
			button_apply.set_sensitive(False)

	# On checkbutton activate
	def on_checkbutton_click(self, check, button_apply, iconview):
		if check.get_active() == True:
			button_apply.set_sensitive(True)
		else:
			liste = iconview.get_selected_items()
			if len(liste) != 0:
				button_apply.set_sensitive(True)
			else:
				button_apply.set_sensitive(False)


	## Constructor
	def __init__(self, directory, images_paths):

		self.images_paths = images_paths
		self.directory = directory

		window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		window.connect("delete_event", self.evnmt_delete)
		window.connect("destroy", self.destroy)
		
		window.set_title("Wallpabox")
		window.set_border_width(10)
		window.set_size_request(650, 500)

		# Alignments
		align_panel = gtk.Alignment(0,0.5,0,0)
		align_buttom = gtk.Alignment(0.5,0,0,0)
		align_left = gtk.Alignment(0,0,0,0)


		# Boxes
		vbox = gtk.VBox(False, 0)
		vbox_panel = gtk.VBox(False, 0)
		hbox = gtk.HBox(False, 0)
		button_box = gtk.VBox(False, 0)
		hbox_bottom = gtk.HBox(False, 0)

		# Label
		label = gtk.Label()
		label.set_markup(_("<b>Wallpapers</b>"))
		vbox.pack_start(label, False, True, 0)


		# Frame
		frame = gtk.Frame()
		frame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
		vbox.pack_start(frame, True, True, 5)


		# Quit button
		button_quit = gtk.Button(stock=gtk.STOCK_QUIT)   
		button_quit.connect_object("clicked", gtk.Widget.destroy, window)
		
		vbox_panel.pack_end(button_quit, False, False, 5)	


		# File selection widget
		filesel = gtk.FileChooserDialog(_("Select your or yours wallpapers"))			

		filesel.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		filesel.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)
		
		filesel.set_select_multiple(True)

		# Filter
		filter_image = gtk.FileFilter()
		filter_image.set_name(_("Image files"))
		filter_image.add_mime_type("image/jpeg")
		filter_image.add_mime_type("image/png")
			
		filesel.add_filter(filter_image);


		# Organize boxes
		align_panel.add(button_box)
		vbox_panel.pack_start(align_panel, True, True, 5)
		hbox.pack_start(vbox, True, True, 5)
		hbox.pack_start(vbox_panel, False, False, 5)
		window.add(hbox)


		# ScrolledWindow
		scwindow = gtk.ScrolledWindow()
		scwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		frame.add(scwindow)

		# Add thumbnails to the main window
		iconview = gtk.IconView()
		iconview.set_selection_mode(gtk.SELECTION_MULTIPLE)
		self.model = gtk.ListStore(gtk.gdk.Pixbuf)

		for path in images_paths:
			try:
				image = gtk.gdk.pixbuf_new_from_file_at_size(path, 96, 96)
				self.model.append([image])					
			except:
				print _("Error : %s is not a correct file.") % path

		iconview.set_model(self.model)
		iconview.set_pixbuf_column(0)		

		scwindow.add(iconview)

		# Drag and drop
		iconview.drag_dest_set(gtk.DEST_DEFAULT_ALL,[("text/uri-list", 0, 0)], gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK | gtk.gdk.ACTION_MOVE)
		iconview.connect("drag_data_received", self.receiveCallback)

		# File selection button
		filew = gtk.Button(stock=gtk.STOCK_ADD)
		filew.connect("clicked", self.select_file, filesel)
		button_box.pack_start(filew, False, False, 5)

		# Remove button
		button_remove = gtk.Button(stock=gtk.STOCK_REMOVE)
		button_remove.set_sensitive(False)
		button_remove.connect("clicked", self.on_remove_click, iconview)
		button_box.pack_start(button_remove, False, False, 5)

		# Choice combobox
		self.combo = gtk.combo_box_new_text() # DO NOT CHANGE ORDER
		self.combo.append_text(_("Fullscreen"))
		self.combo.append_text(_("Centered"))
		self.combo.append_text(_("Tiled"))
		self.combo.append_text(_("Maximized"))
		self.combo.set_active(0)
		hbox_bottom.pack_start(self.combo, False, False)

		# gtk.CheckButton 
		self.randomcheck = gtk.CheckButton(_("Random wallpapers"))
		hbox_bottom.pack_start(self.randomcheck, False, False)

		# Apply button
		button_apply = gtk.Button(stock=gtk.STOCK_APPLY)
		button_apply.set_sensitive(False) 
		align_buttom.add(button_apply)
		hbox_bottom.pack_start(align_buttom, True, False, 0)
		vbox.pack_end(hbox_bottom, False, True, 5)

		button_apply.connect("clicked", self.on_apply_click, iconview)

		iconview.connect("selection-changed", self.on_selection_changed, button_apply, button_remove)
		iconview.connect("item-activated", self.on_double_click, button_apply)
		self.combo.connect("changed",self.on_combo_change, button_apply, iconview)
		self.randomcheck.connect("clicked", self.on_checkbutton_click, button_apply, iconview)
		# Show widgets
		window.show_all()



def main():
	gtk.main()
	return 0

######################################################

if __name__ == "__main__":
	
	# Create a directory if it does not exist
	directory = os.path.expanduser('~')+"/.wallpabox/"

	if not os.path.exists(directory):
		print _("Create a new config directory")
		os.mkdir(directory, 0777)

	# Get all images from the config directory
	images_paths = os.listdir(directory)
	images_paths_final = []
	for path in images_paths:
		if(path.lower().endswith('jpg') or path.lower().endswith('jpeg') or path.lower().endswith('png') or path.lower().endswith('gif') or p.lower().endswith('bmp')):
			images_paths_final.append(directory+path)

	ChangeWallpaper(directory, images_paths_final)
	main()

